#pragma once
//#include <Winsock2.h>

/* #######################################################################
* B2 Header
* ####################################################################### */
#if 1
typedef struct {
	unsigned char Key[4];  /* 00 00 01 B2 */
	unsigned char  Type;
	unsigned char Stream_id;
	unsigned char Ext_b2_len;	// 1: length of the ext. b2 private data appended to B2 Frame
	unsigned char Rsvd;
	unsigned int  Len;
} B2_HEADER, ACTI_HEADER;
#else
typedef struct {
	unsigned char Key[4];  /* 00 00 01 B2 */
	unsigned int  Type;
	unsigned int  Len;
} B2_HEADER, ACTI_HEADER;
#endif

#define B2_HEADER_LEN                 sizeof(B2_HEADER)

typedef struct {
	B2_HEADER      Head;
	struct timeval TS;
	unsigned char  Rsvd[8];
} AUDIO_B2_FRAME;

#define AUDIO_B2_FRAME_LEN		sizeof(AUDIO_B2_FRAME)

/* #######################################################################
* Private data followed by B2 header
* ####################################################################### */
/* definitions of VLoss in PRIVATE_DATA */
#define B2_VIDEO_LOSS_NOT_FOUND     1       /* video ok */
#define B2_VIDEO_LOSS_FOUND         0       /* video loss */
typedef struct {
	//time_t         Time;
	unsigned long  Time;
	unsigned char  TimeZone;   /* mapping the TIME_ZONE in conf file. 0:-12, ..., 24:+13 */
	unsigned char  VLoss;      /* 0: video loss, 1 : video ok */
	unsigned char  Motion;
	unsigned char  DIO;			/* for DIs, 0: DI triggered. 1: no triggered */
	unsigned int   Cnt;
	unsigned char  Resolution;  /* mapping the VIDEO_RESOLUTION in cond file. 0:N720x480, ... */
	unsigned char  BitRate;     /* mapping the VIDEO_BITRATE in cond file. 0:28K, ... */
	unsigned char  FpsMode;     /* mapping the VIDEO_FPS in cond file. 0:MODE1(constant), 1:0:MODE2 */
	unsigned char  FpsNum;      /* in constant FPS mode, it indicates the video server's constant
									FPS number, i.e. atoi(VIDEO_FPS_NUM).
									in variable FPS mode, in indicates the variable FPS number which
									was requested by the TCP host. If it is not in TCP, it indicates
									the variable FPS number, i.e. atoi(VIDEO_VARIABLE_FPS) */
	struct timeval TS;
	unsigned char  Rsvd[8];
} PRIVATE_DATA;

#define PRIVATE_DATA_LEN         sizeof(PRIVATE_DATA_LEN)

typedef struct {
	B2_HEADER      Head;
	PRIVATE_DATA   Data;
} B2_FRAME;

#define B2_FRAME_LEN                    sizeof(B2_FRAME)
#define MAX_RESERVED_AUDIO_HEADER       AUDIO_HEADER_LEN

/* #######################################################################
* definitions of status in TCP_AUTHEN_REPLY
* ####################################################################### */
#define TCP_AUTHEN_SUCCESS              0x00
#define TCP_AUTHEN_ERROR                0x01

typedef struct {
	char Status;
	char Rsvd[3];
	int  Sock;
	char CameraName[120];
} TCP_AUTHEN_REPLY;

#define NTSC_720x480	0
#define NTSC_640x480	64			///< #8# - NTSC - 160 x 112
#define NTSC_1280x720	65			///< #9# - NTSC - 1280 x 720
#define NTSC_1280x900	66			///< #10# - NTSC - 1280 x 960
#define NTSC_1280x1024	67			///< #11# - NTSC - 1280 x 1024
#define NTSC_1920x1080	68			///< #12# - NTSC - 1920 x 1080

#define FRAME_TYPE_UNKNOWN	0
#define FRAME_TYPE_VIDEO_I	2
#define FRAME_TYPE_VIDEO_P	3
#define FRAME_TYPE_B2		4

#define FRAME_TYPE_VEDIO_MP4V	1
#define FRAME_TYPE_AUDIO_PCM	2
#define FRAME_TYPE_VEDIO_JPEG	4
#define FRAME_TYPE_VEDIO_H264	5

inline int fnGetVideoFrameType( const unsigned char *buf, int *Type = NULL )
{
	int type = FRAME_TYPE_UNKNOWN;
	if((buf[0]==0x00)&&(buf[1]==0x00)&&(buf[2]==0x01)&&(buf[3]==0xb6))
	{
		type = FRAME_TYPE_VIDEO_P;
	}
	else if((buf[0]==0x00)&&(buf[1]==0x00)&&(buf[2]==0x01)&&(buf[3]==0xb0))
	{
		type = FRAME_TYPE_VIDEO_I;
	}
	else if((buf[0]==0x00)&&(buf[1]==0x00)&&(buf[2]==0x01)&&(buf[3]==0xb2))
	{
		type = FRAME_TYPE_B2;
	}
	else
	{
		type = FRAME_TYPE_UNKNOWN;
	}

	if( Type )
		*Type = type;

	return type;
}