#ifndef __DEFCODE_H__
#define __DEFCODE_H__
/** \brief All the definition code here.
*
*/

// Import & Export definition
#ifdef __DLL_IMPORT__
#define DLL_API extern "C" __declspec(dllimport)	/**< Import definition */
#elif defined(__DLL_EXPORT__) 
#define DLL_API extern "C" __declspec(dllexport)	/**< Export definition */
#endif 

#define DEBUGGING 0

#define PLAYBACK_VERSION	"1.0.0.8"	/** Version of play back */

// TCP version
#define TCP_VERSION_10	0x11	/**< TCP Version 1.0 */
#define TCP_VERSION_20	0x22	/**< TCP Version 2.0 */

// Video Control Code
#define TCP10_VIDEO_CONTROL_CODE    0x00010022	/**< Control code for TCP 1.0 */

// Temp string for TCP 10 header to compare
#define Temp "\x41\x43\x54\x69"	/**<  String for TCP 10 header to compare */

// Message type
#define MSG_TYPE_UNKNOW		0x00	/**< Type Unknow */
#define MSG_TYPE_VIDEO_20	0x01	/**< Video data type for TCP 2.0 */
#define MSG_TYPE_AUDIO_20	0x02	/**< Audio data type for TCP 2.0 */
#define MSG_TYPE_10			0x01	/**< Data type for TCP 1.0 */
#define MSG_TYPE_EOF		0x99	/**< End of file */

// Frame type
#define FRAME_TYPE_UNKNOW	0x00	/**< Type unknow */
#define FRAME_TYPE_I		0x01	/**< I Frame */
#define FRAME_TYPE_P		0X02	/**< P Frame */

// Error codes
#define PLAYBACK_SUCCESS	1	/**< Success code for play back */
#define PLAYBACK_ERROR_FAIL	0	/**< Fail code for play back */

// Ohter constant
#define MAX_FILE_LEN	512			/**< Max. file length the applicatoin will take. */
#define HEADER_LEN		64			/**< Header length (Big enough to cover header) */
#define DATA_LEN		256*1000-64	/**< Data length (Big enough to cover data) */
#define FRAME_LEN		256*1000	/**< Frame length = Header + Data length */

// Header flags
#define FLAG_GENERAL_HEADER		0x010000	/**< General Header 010000 : 3 bytes */ 
#define FLAG_GOP_HEADER			0xB3010000	/**< GOP Header */
#define FLAG_FRAME_HEADER		0xB6010000	/**< Frame Header */
#define FLAG_CUSTOMER_HEADER	0xB2010000	/**< Constant value for customer header to compare */

#define TCP10_CUSTOMER_DATA_SIZE 16					/**< Customer data length B2 data */
#define TCP20_B0_DATA_SIZE 5						/**< size of B0 including 0x000001B0.  B0 Header */
#define TCP20_B5_DATA_SIZE 5						/**< size of B5 including 0x000001B5.  B5 Header */
#define TCP20_00_DATA_SIZE 21						/**< sequence header including 0x00000100 */
#define TCP20_VIDEO_HEADER_DATA_SIZE	32			/**< size of struct video header */
#define TCP1020_HEADER_SIZE	12						/**< size of 10 & 20's header */

#define TEMP_BUFFER_SZE 1024						/*< Constant for temporary buffer size */

static unsigned char VariableHeader[6][21]= 
{
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xBA,0x98,0x60,0xFA,0x62,0xD0,0x87,0x82,0x83,0x07}, //N720x480  
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xBA,0x98,0x60,0xFA,0x61,0x60,0x83,0xC2,0x83,0x07}, //N352x240
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xBA,0x98,0x60,0xFA,0x60,0xA0,0x81,0xC2,0x83,0x07}, //N160x122
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xB0,0xD4,0x60,0xFA,0x62,0xD0,0x89,0x02,0x83,0x07}, //P720x576
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xB0,0xD4,0x60,0xFA,0x61,0x60,0x84,0x82,0x83,0x07}, //P352x288
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xB0,0xD4,0x60,0xFA,0x60,0xB0,0x82,0x42,0x83,0x07}  //P176x144
};

static unsigned char ConstantHeader2[180][21]= 
{
	/* N720x480 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x81,0xf4,0x7f,0x4c,0x5a,0x10,0xf0,0x50,0x60,0x00}, //1
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x83,0xe8,0x6f,0xa6,0x2d,0x08,0x78,0x28,0x30,0x7f}, //2
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x85,0xdc,0x67,0xd3,0x16,0x84,0x3c,0x14,0x18,0x3f}, //3
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x87,0xd0,0x67,0xd3,0x16,0x84,0x3c,0x14,0x18,0x3f}, //4
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb2,0xc8,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb4,0xbc,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb6,0xb0,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb8,0xa4,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xba,0x98,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //30
	/* N352x240 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x81,0xf4,0x7f,0x4c,0x2c,0x10,0x78,0x50,0x60,0x00}, //1
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x83,0xe8,0x6f,0xa6,0x16,0x08,0x3c,0x28,0x30,0x7f}, //2
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x85,0xdc,0x67,0xd3,0x0b,0x04,0x1e,0x14,0x18,0x3f}, //3
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x87,0xd0,0x67,0xd3,0x0b,0x04,0x1e,0x14,0x18,0x3f}, //4
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb2,0xc8,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb4,0xbc,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb6,0xb0,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb8,0xa4,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xba,0x98,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //30
	/* N160x112 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x81,0xf4,0x7f,0x4c,0x14,0x10,0x38,0x50,0x60,0x00}, //1
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x83,0xe8,0x6f,0xa6,0x0a,0x08,0x1c,0x28,0x30,0x7f}, //2
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x85,0xdc,0x67,0xd3,0x05,0x04,0x0e,0x14,0x18,0x3f}, //3
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x87,0xd0,0x67,0xd3,0x05,0x04,0x0e,0x14,0x18,0x3f}, //4
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb2,0xc8,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb4,0xbc,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb6,0xb0,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb8,0xa4,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xba,0x98,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //30
	/* P720x576 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x81,0xf4,0x7f,0x4c,0x5a,0x11,0x20,0x50,0x60,0x00}, //1
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x83,0xe8,0x6f,0xa6,0x2d,0x08,0x90,0x28,0x30,0x7f}, //2
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x85,0xdc,0x67,0xd3,0x16,0x84,0x48,0x14,0x18,0x3f}, //3
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x87,0xd0,0x67,0xd3,0x16,0x84,0x48,0x14,0x18,0x3f}, //4
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //30
	/* P352x288 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x81,0xf4,0x7f,0x4c,0x2c,0x10,0x90,0x50,0x60,0x00}, //1
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x83,0xe8,0x6f,0xa6,0x16,0x08,0x48,0x28,0x30,0x7f}, //2
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x85,0xdc,0x67,0xd3,0x0b,0x04,0x24,0x14,0x18,0x3f}, //3
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x87,0xd0,0x67,0xd3,0x0b,0x04,0x24,0x14,0x18,0x3f}, //4
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //30
	/* P176x144 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x81,0xf4,0x7f,0x4c,0x16,0x10,0x48,0x50,0x60,0x00}, //1
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x83,0xe8,0x6f,0xa6,0x0b,0x08,0x24,0x28,0x30,0x7f}, //2
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x85,0xdc,0x67,0xd3,0x05,0x84,0x12,0x14,0x18,0x3f}, //3
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x87,0xd0,0x67,0xd3,0x05,0x84,0x12,0x14,0x18,0x3f}, //4
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}  //30
};

static unsigned char ConstantHeader[180][21]= 
{
	/* N720x480 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x8b,0x42,0x1e,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc5,0xa1,0x0f,0x05,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb2,0xc8,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb4,0xbc,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb6,0xb0,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb8,0xa4,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xba,0x98,0x60,0xfa,0x62,0xd0,0x87,0x82,0x83,0x07}, //30
	/* N352x240 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x85,0x82,0x0f,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc2,0xc1,0x07,0x85,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb2,0xc8,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb4,0xbc,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb6,0xb0,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb8,0xa4,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xba,0x98,0x60,0xfa,0x61,0x60,0x83,0xc2,0x83,0x07}, //30
	/* N160x112 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x82,0x82,0x07,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc1,0x41,0x03,0x85,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb2,0xc8,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb4,0xbc,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb6,0xb0,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb8,0xa4,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xba,0x98,0x60,0xfa,0x60,0xa0,0x81,0xc2,0x83,0x07}, //30
	/* P720x576 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x8b,0x42,0x24,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc5,0xa1,0x12,0x05,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x62,0xd0,0x89,0x02,0x83,0x07}, //30
	/* P352x288 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x85,0x82,0x12,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc2,0xc1,0x09,0x05,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x61,0x60,0x84,0x82,0x83,0x07}, //30
	/* P176x144 */
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x89,0xc4,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //5
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8b,0xb8,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //6
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8d,0xac,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //7
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x8f,0xa0,0x63,0xe9,0x82,0xc2,0x09,0x0a,0x0c,0x1f}, //8
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x91,0x94,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //9
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x93,0x88,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //10
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x95,0x7c,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //11
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x97,0x70,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //12
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x99,0x64,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //13
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9b,0x58,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //14
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9d,0x4c,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //15
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0x9f,0x40,0x61,0xf4,0xc1,0x61,0x04,0x85,0x06,0x0f}, //16
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa1,0x34,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //17
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa3,0x28,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //18
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa5,0x1c,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //19
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa7,0x10,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //20
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xa9,0x04,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //21
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xaa,0xf8,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //22
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xac,0xec,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //23
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xae,0xe0,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //24
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //25
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //26
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //27
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //28
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}, //29
	{0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x20,0x00,0xC8,0x88,0xb0,0xd4,0x60,0xfa,0x60,0xb0,0x82,0x42,0x83,0x07}  //30
};

//============== Common Configuration Structure & Definition ==============

typedef void ( WINAPI *CONTROL_DATA_CALLBACK	)( DWORD UserParam, BYTE* buf, DWORD len );

#define CONTACT_TYPE_UNICAST_WOC_PREVIEW		1
#define CONTACT_TYPE_MULTICAST_WOC_PREVIEW		2
#define CONTACT_TYPE_RTSP_PREVIEW				3
#define CONTACT_TYPE_CONTROL_ONLY				4
#define CONTACT_TYPE_UNICAST_PREVIEW			5
#define CONTACT_TYPE_MULTICAST_PREVIEW			6
#define CONTACT_TYPE_PLAYBACK					7
#define CONTACT_TYPE_CARD_PREVIEW				8

#define NET_TVNTSC	0
#define NET_TVPAL	1

#define ERROR_RETURN	-1	/**< Error return code */


typedef struct structural_MEDIA_CONNECTION_CONFIG
{		
	int	ContactType;

	unsigned char	ChannelNumber;	// For URL Command CHANNEL tag, when set it to 0, URL command won't bring CHANNEL tag, or the URL command will bring CHANNEL=ChannelNumber tag  )
	unsigned char	TCPVideoStreamID;	// 0 based to specify video track, value 0 to 255 for 1 to 256 video track
	unsigned char	RTPVideoTrackNumber;	// set it to 0, ARTP will use 1st video track,  1 to 255 is for specify video track
	unsigned char	RTPAudioTrackNumber;	// set it to 0, ARTP will use 1st audio track,  1 to 255 is for specify audio track

	char			UniCastIP[16];
	char			MultiCastIP[16];
	char			PlayFileName[256];
	char			UserID[64];
	char			Password[64];
	unsigned long	RegisterPort;
	unsigned long	StreamingPort;
	unsigned long	ControlPort;
	unsigned long	MultiCastPort;
	unsigned long	SearchPortC2S;
	unsigned long	SearchPortS2C;
	unsigned long	HTTPPort;
	unsigned long	RTSPPort;
	unsigned long	Reserved1;
	unsigned long	Reserved2;

	unsigned short	ConnectTimeOut;
	unsigned short	EncryptionType;
}MEDIA_CONNECTION_CONFIG;

typedef struct
{
	DWORD dwTvStandard;			// 0:NTSC 1:PAL
	DWORD dwVideoResolution;	// See the definition above
	DWORD dwBitsRate;			// See the definition above
	DWORD dwVideoBrightness;	// 0 ~ 100 : Low ~ High
	DWORD dwVideoContrast;		// 0 ~ 100 : Low ~ High
	DWORD dwVideoSaturation;	// 0 ~ 100 : Low ~ High
	DWORD dwVideoHue;			// 0 ~ 100 : Low ~ High
	DWORD dwFps;				// 0 ~ 30 frame pre second
} MEDIA_VIDEO_CONFIG;

typedef struct
{
	DWORD dwCommandType;
	char* pCommand;
	int	  nCommandLength;
	char* pResult;
	int   nResultLength;
} MEDIA_COMMAND;

typedef struct structural_MEDIA_MOTION_INFO
{
	DWORD dwEnable;
	DWORD dwRangeCount;
	DWORD dwRange[3][4];
	DWORD dwSensitive[3];
} MEDIA_MOTION_INFO;
//=========================================================================

// Debug message level define
#define DEBUG_LEVEL_NO_DEBUG_MESSAGE            0
#define DEBUG_LEVEL_FUNCTION_TRACE              1
#define DEBUG_LEVEL_DETAIL                      2

// Jo_20060524 - Send command request ID(S)
#define  PLY_REQ_SET_POSITION   0xBAC00001			
#define  PLY_REQ_SET_TIMECODE   0xBAC00002
// Jo_20060524 - Send command request ID(E)

// Jo_20060525 - I frame information going to add to link list(S)
#define METHOD_FILE_POS	1
#define METHOD_FILE_TIME_CODE	2

typedef struct IFrameInfo_t
{
	DWORD dwFilePos;	/**< File position */
	DWORD dwTimeCode;	/**< Time code */
}IFrameInfo;
// Jo_20060525 - I frame information going to add to link list(E)

#endif // __DEFCODE_H__